@@ -180,7 +180,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
         const ProStringList &incs = project->values("INCLUDEPATH");
         for(int i = 0; i < incs.size(); ++i) {
             const ProString &inc = incs.at(i);
-            if (inc.isEmpty())
+            if (inc.isEmpty() || inc == "/usr/include")
                 continue;
 
             if (!isystem.isEmpty() && isSystemInclude(inc.toQString()))
