@@ -1,32 +1,38 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 Digia Plc and/or its subsidiary(-ies).
-** Copyright (C) 2014 Intel Corporation
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2016 Intel Corporation.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,25 +47,20 @@
 
 #ifdef QT_BUILD_QMAKE
 QT_BEGIN_NAMESPACE
-extern QString qt_libraryInfoFile();
+extern QString qmake_libraryInfoFile();
 QT_END_NAMESPACE
-#elif defined(QT_BOOTSTRAPPED)
-QString qt_libraryInfoFile()
-{
-    QString qmakeAbsoluteLocation = QLatin1String(QT_QMAKE_LOCATION);
-    if (!qmakeAbsoluteLocation.isEmpty())
-        return QDir(QFileInfo(qmakeAbsoluteLocation).absolutePath()).filePath(QLatin1String("qt.conf"));
-    return QString();
-}
 #else
 # include "qcoreapplication.h"
 #endif
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
 #  include "private/qcore_mac_p.h"
 #endif
 
-#include "qconfig.cpp"
+#ifndef QT_BUILD_QMAKE_BOOTSTRAP
+# include "qconfig.cpp"
+#endif
+
 #include "archdetect.cpp"
 
 QT_BEGIN_NAMESPACE
@@ -71,11 +72,16 @@ extern void qDumpCPUFeatures(); // in qsimd.cpp
 struct QLibrarySettings
 {
     QLibrarySettings();
+    void load();
+
     QScopedPointer<QSettings> settings;
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
+    bool haveDevicePaths;
     bool haveEffectiveSourcePaths;
     bool haveEffectivePaths;
     bool havePaths;
+#else
+    bool reloadOnQAppAvailable;
 #endif
 };
 Q_GLOBAL_STATIC(QLibrarySettings, qt_library_settings)
@@ -84,29 +90,53 @@ class QLibraryInfoPrivate
 {
 public:
     static QSettings *findConfiguration();
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
+    static void reload()
+    {
+        if (qt_library_settings.exists())
+            qt_library_settings->load();
+    }
     static bool haveGroup(QLibraryInfo::PathGroup group)
     {
         QLibrarySettings *ls = qt_library_settings();
         return ls ? (group == QLibraryInfo::EffectiveSourcePaths
                      ? ls->haveEffectiveSourcePaths
                      : group == QLibraryInfo::EffectivePaths
-                       ? ls->haveEffectivePaths : ls->havePaths) : false;
+                       ? ls->haveEffectivePaths
+                       : group == QLibraryInfo::DevicePaths
+                         ? ls->haveDevicePaths
+                         : ls->havePaths) : false;
     }
 #endif
     static QSettings *configuration()
     {
         QLibrarySettings *ls = qt_library_settings();
-        return ls ? ls->settings.data() : 0;
+        if (ls) {
+#ifndef QT_BUILD_QMAKE
+            if (ls->reloadOnQAppAvailable && QCoreApplication::instance() != 0)
+                ls->load();
+#endif
+            return ls->settings.data();
+        } else {
+            return 0;
+        }
     }
 };
 
 static const char platformsSection[] = "Platforms";
 
 QLibrarySettings::QLibrarySettings()
-    : settings(QLibraryInfoPrivate::findConfiguration())
 {
-#ifndef QT_BOOTSTRAPPED
+    load();
+}
+
+void QLibrarySettings::load()
+{
+    // If we get any settings here, those won't change when the application shows up.
+    settings.reset(QLibraryInfoPrivate::findConfiguration());
+#ifndef QT_BUILD_QMAKE
+    reloadOnQAppAvailable = (settings.data() == 0 && QCoreApplication::instance() == 0);
+    bool haveDevicePaths;
     bool haveEffectivePaths;
     bool havePaths;
 #endif
@@ -114,7 +144,8 @@ QLibrarySettings::QLibrarySettings()
         // This code needs to be in the regular library, as otherwise a qt.conf that
         // works for qmake would break things for dynamically built Qt tools.
         QStringList children = settings->childGroups();
-#ifdef QT_BOOTSTRAPPED
+        haveDevicePaths = children.contains(QLatin1String("DevicePaths"));
+#ifdef QT_BUILD_QMAKE
         haveEffectiveSourcePaths = children.contains(QLatin1String("EffectiveSourcePaths"));
 #else
         // EffectiveSourcePaths is for the Qt build only, so needs no backwards compat trickery.
@@ -122,13 +153,15 @@ QLibrarySettings::QLibrarySettings()
 #endif
         haveEffectivePaths = haveEffectiveSourcePaths || children.contains(QLatin1String("EffectivePaths"));
         // Backwards compat: an existing but empty file is claimed to contain the Paths section.
-        havePaths = (!haveEffectivePaths && !children.contains(QLatin1String(platformsSection)))
+        havePaths = (!haveDevicePaths && !haveEffectivePaths
+                     && !children.contains(QLatin1String(platformsSection)))
                     || children.contains(QLatin1String("Paths"));
-#ifndef QT_BOOTSTRAPPED
+#ifndef QT_BUILD_QMAKE
         if (!havePaths)
             settings.reset(0);
 #else
     } else {
+        haveDevicePaths = false;
         haveEffectiveSourcePaths = false;
         haveEffectivePaths = false;
         havePaths = false;
@@ -138,34 +171,36 @@ QLibrarySettings::QLibrarySettings()
 
 QSettings *QLibraryInfoPrivate::findConfiguration()
 {
-    QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
-#ifdef QT_BOOTSTRAPPED
-    if(!QFile::exists(qtconfig))
-        qtconfig = qt_libraryInfoFile();
-#else
-    if (!QFile::exists(qtconfig) && QCoreApplication::instance()) {
-#ifdef Q_OS_MAC
-        CFBundleRef bundleRef = CFBundleGetMainBundle();
-        if (bundleRef) {
-            QCFType<CFURLRef> urlRef = CFBundleCopyResourceURL(bundleRef,
-                                                               QCFString(QLatin1String("qt.conf")),
-                                                               0,
-                                                               0);
-            if (urlRef) {
-                QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
-                qtconfig = QDir::cleanPath(path);
-            }
-        }
-        if (qtconfig.isEmpty())
-#endif
-            {
-                QDir pwd(QCoreApplication::applicationDirPath());
-                qtconfig = pwd.filePath(QLatin1String("qt.conf"));
-            }
-    }
-#endif
+#ifdef QT_BUILD_QMAKE
+    QString qtconfig = qmake_libraryInfoFile();
     if (QFile::exists(qtconfig))
         return new QSettings(qtconfig, QSettings::IniFormat);
+#else
+    QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
+    if (QFile::exists(qtconfig))
+        return new QSettings(qtconfig, QSettings::IniFormat);
+#ifdef Q_OS_DARWIN
+    CFBundleRef bundleRef = CFBundleGetMainBundle();
+    if (bundleRef) {
+        QCFType<CFURLRef> urlRef = CFBundleCopyResourceURL(bundleRef,
+                                                           QCFString(QLatin1String("qt.conf")),
+                                                           0,
+                                                           0);
+        if (urlRef) {
+            QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
+            qtconfig = QDir::cleanPath(path);
+            if (QFile::exists(qtconfig))
+                return new QSettings(qtconfig, QSettings::IniFormat);
+        }
+    }
+#endif
+    if (QCoreApplication::instance()) {
+        QDir pwd(QCoreApplication::applicationDirPath());
+        qtconfig = pwd.filePath(QLatin1String("qt.conf"));
+        if (QFile::exists(qtconfig))
+            return new QSettings(qtconfig, QSettings::IniFormat);
+    }
+#endif
     return 0;     //no luck
 }
 
@@ -189,7 +224,7 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
     \sa QSysInfo, {Using qt.conf}
 */
 
-#ifndef QT_BOOTSTRAPPED
+#ifndef QT_BUILD_QMAKE
 
 /*!
     \internal
@@ -202,42 +237,44 @@ QLibraryInfo::QLibraryInfo()
 { }
 
 /*!
-  Returns the person to whom this build of Qt is licensed.
-
-  \sa licensedProducts()
+  \deprecated
+  This function used to return the person to whom this build of Qt is licensed, now returns an empty string.
 */
 
+#if QT_DEPRECATED_SINCE(5, 8)
 QString
 QLibraryInfo::licensee()
 {
-    const char * volatile str = QT_CONFIGURE_LICENSEE;
-    return QString::fromLocal8Bit(str);
+    return QString();
 }
+#endif
 
 /*!
-  Returns the products that the license for this build of Qt has access to.
-
-  \sa licensee()
+  \deprecated
+  This function used to return the products that the license for this build of Qt has access to, now returns an empty string.
 */
 
+#if QT_DEPRECATED_SINCE(5, 8)
 QString
 QLibraryInfo::licensedProducts()
 {
-    const char * volatile str = QT_CONFIGURE_LICENSED_PRODUCTS;
-    return QString::fromLatin1(str);
+    return QString();
 }
+#endif
 
 /*!
     \since 4.6
-    Returns the installation date for this build of Qt. The install date will
-    usually be the last time that Qt sources were configured.
+    \deprecated
+    This function used to return the installation date for this build of Qt, but now returns a constant date.
 */
 #ifndef QT_NO_DATESTRING
+#if QT_DEPRECATED_SINCE(5, 5)
 QDate
 QLibraryInfo::buildDate()
 {
     return QDate::fromString(QString::fromLatin1(qt_configure_installation + 12), Qt::ISODate);
 }
+#endif
 #endif //QT_NO_DATESTRING
 
 #if defined(Q_CC_INTEL) // must be before GNU, Clang and MSVC because ICC/ICL claim to be them
@@ -276,6 +313,8 @@ QLibraryInfo::buildDate()
 #  else
 #    define COMPILER_STRING "Clang " __clang_version__
 #  endif
+#elif defined(Q_CC_GHS)
+#  define COMPILER_STRING "GHS " QT_STRINGIFY(__GHS_VERSION_NUMBER)
 #elif defined(Q_CC_GNU)
 #  define COMPILER_STRING "GCC " __VERSION__
 #elif defined(Q_CC_MSVC)
@@ -287,6 +326,10 @@ QLibraryInfo::buildDate()
 #    define COMPILER_STRING "MSVC 2012"
 #  elif _MSC_VER < 1900
 #    define COMPILER_STRING "MSVC 2013"
+#  elif _MSC_VER < 1910
+#    define COMPILER_STRING "MSVC 2015"
+#  elif _MSC_VER < 2000
+#    define COMPILER_STRING "MSVC 2017"
 #  else
 #    define COMPILER_STRING "MSVC _MSC_VER " QT_STRINGIFY(_MSC_VER)
 #  endif
@@ -333,8 +376,21 @@ QLibraryInfo::isDebugBuild()
 #endif
 }
 
+#ifndef QT_BOOTSTRAPPED
+/*!
+    \since 5.8
+    Returns the version of the Qt library.
+
+    \sa qVersion()
+*/
+QVersionNumber QLibraryInfo::version() Q_DECL_NOTHROW
+{
+    return QVersionNumber(QT_VERSION_MAJOR, QT_VERSION_MINOR, QT_VERSION_PATCH);
+}
 #endif // QT_BOOTSTRAPPED
 
+#endif // QT_BUILD_QMAKE
+
 /*
  * To add a new entry in QLibrary::LibraryLocation, add it to the enum above the bootstrapped values and:
  * - add its relative path in the qtConfEntries[] array below
@@ -365,35 +421,46 @@ static const struct {
     { "Translations", "translations" }, // should be ${Data}/translations
     { "Examples", "examples" },
     { "Tests", "tests" },
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
     { "Sysroot", "" },
-    { "HostPrefix", "" },
+    { "SysrootifyPrefix", "" },
     { "HostBinaries", "bin" },
     { "HostLibraries", "lib" },
     { "HostData", "." },
     { "TargetSpec", "" },
     { "HostSpec", "" },
+    { "HostPrefix", "" },
 #endif
 };
 
+#ifdef QT_BUILD_QMAKE
+void QLibraryInfo::reload()
+{
+    QLibraryInfoPrivate::reload();
+}
+#endif
+
 /*!
   Returns the location specified by \a loc.
-
 */
 QString
 QLibraryInfo::location(LibraryLocation loc)
 {
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE // ends inside rawLocation !
     QString ret = rawLocation(loc, FinalPaths);
 
     // Automatically prepend the sysroot to target paths
-    if ((loc < SysrootPath || loc > LastHostPath) && qt_sysrootify_prefix[12] == 'y') {
+    if (loc < SysrootPath || loc > LastHostPath) {
         QString sysroot = rawLocation(SysrootPath, FinalPaths);
-        if (!sysroot.isEmpty() && ret.length() > 2 && ret.at(1) == QLatin1Char(':')
-            && (ret.at(2) == QLatin1Char('/') || ret.at(2) == QLatin1Char('\\')))
-            ret.replace(0, 2, sysroot); // Strip out the drive on Windows targets
-        else
-            ret.prepend(sysroot);
+        if (!sysroot.isEmpty()
+                && QVariant::fromValue(rawLocation(SysrootifyPrefixPath, FinalPaths)).toBool()) {
+            if (ret.length() > 2 && ret.at(1) == QLatin1Char(':')
+                   && (ret.at(2) == QLatin1Char('/') || ret.at(2) == QLatin1Char('\\'))) {
+                ret.replace(0, 2, sysroot); // Strip out the drive on Windows targets
+            } else {
+                ret.prepend(sysroot);
+            }
+        }
     }
 
     return ret;
@@ -402,38 +469,30 @@ QLibraryInfo::location(LibraryLocation loc)
 QString
 QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
 {
-#else
-# define rawLocation(loca, group) location(loca)
-# define group dummy
-#endif
+#endif // QT_BUILD_QMAKE, started inside location !
     QString ret;
-#ifdef QT_BOOTSTRAPPED
+    bool fromConf = false;
+#ifndef QT_NO_SETTINGS
+#ifdef QT_BUILD_QMAKE
     // Logic for choosing the right data source: if EffectivePaths are requested
     // and qt.conf with that section is present, use it, otherwise fall back to
     // FinalPaths. For FinalPaths, use qt.conf if present and contains not only
     // [EffectivePaths], otherwise fall back to builtins.
     // EffectiveSourcePaths falls back to EffectivePaths.
-    if (!QLibraryInfoPrivate::haveGroup(group)
-        && !(group == EffectiveSourcePaths
-             && (group = EffectivePaths, QLibraryInfoPrivate::haveGroup(group)))
-        && !(group == EffectivePaths
-             && (group = FinalPaths, QLibraryInfoPrivate::haveGroup(group))))
-#elif !defined(QT_NO_SETTINGS)
-    if (!QLibraryInfoPrivate::configuration())
+    // DevicePaths falls back to FinalPaths.
+    PathGroup orig_group = group;
+    if (QLibraryInfoPrivate::haveGroup(group)
+        || (group == EffectiveSourcePaths
+            && (group = EffectivePaths, QLibraryInfoPrivate::haveGroup(group)))
+        || ((group == EffectivePaths || group == DevicePaths)
+            && (group = FinalPaths, QLibraryInfoPrivate::haveGroup(group)))
+        || (group = orig_group, false))
+#else
+    if (QLibraryInfoPrivate::configuration())
 #endif
     {
-        const char *path = 0;
-        if (unsigned(loc) < sizeof(qt_configure_prefix_path_strs)/sizeof(qt_configure_prefix_path_strs[0]))
-            path = qt_configure_prefix_path_strs[loc] + 12;
-#ifndef Q_OS_WIN // On Windows we use the registry
-        else if (loc == SettingsPath)
-            path = QT_CONFIGURE_SETTINGS_PATH;
-#endif
+        fromConf = true;
 
-        if (path)
-            ret = QString::fromLocal8Bit(path);
-#ifndef QT_NO_SETTINGS
-    } else {
         QString key;
         QString defaultValue;
         if (unsigned(loc) < sizeof(qtConfEntries)/sizeof(qtConfEntries[0])) {
@@ -450,7 +509,8 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
         if(!key.isNull()) {
             QSettings *config = QLibraryInfoPrivate::configuration();
             config->beginGroup(QLatin1String(
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
+                   group == DevicePaths ? "DevicePaths" :
                    group == EffectiveSourcePaths ? "EffectiveSourcePaths" :
                    group == EffectivePaths ? "EffectivePaths" :
 #endif
@@ -458,79 +518,122 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
 
             ret = config->value(key, defaultValue).toString();
 
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
             if (ret.isEmpty()) {
                 if (loc == HostPrefixPath)
                     ret = config->value(QLatin1String(qtConfEntries[PrefixPath].key),
                                         QLatin1String(qtConfEntries[PrefixPath].value)).toString();
-                else if (loc == TargetSpecPath || loc == HostSpecPath)
-                    ret = QString::fromLocal8Bit(qt_configure_prefix_path_strs[loc] + 12);
+                else if (loc == TargetSpecPath || loc == HostSpecPath || loc == SysrootifyPrefixPath)
+                    fromConf = false;
+                // The last case here is SysrootPath, which can be legitimately empty.
+                // All other keys have non-empty fallbacks to start with.
             }
 #endif
 
-            // expand environment variables in the form $(ENVVAR)
-            int rep;
-            QRegExp reg_var(QLatin1String("\\$\\(.*\\)"));
-            reg_var.setMinimal(true);
-            while((rep = reg_var.indexIn(ret)) != -1) {
-                ret.replace(rep, reg_var.matchedLength(),
-                            QString::fromLocal8Bit(qgetenv(ret.mid(rep + 2,
-                                reg_var.matchedLength() - 3).toLatin1().constData()).constData()));
+            int startIndex = 0;
+            forever {
+                startIndex = ret.indexOf(QLatin1Char('$'), startIndex);
+                if (startIndex < 0)
+                    break;
+                if (ret.length() < startIndex + 3)
+                    break;
+                if (ret.at(startIndex + 1) != QLatin1Char('(')) {
+                    startIndex++;
+                    continue;
+                }
+                int endIndex = ret.indexOf(QLatin1Char(')'), startIndex + 2);
+                if (endIndex < 0)
+                    break;
+                QStringRef envVarName = ret.midRef(startIndex + 2, endIndex - startIndex - 2);
+                QString value = QString::fromLocal8Bit(qgetenv(envVarName.toLocal8Bit().constData()));
+                ret.replace(startIndex, endIndex - startIndex + 1, value);
+                startIndex += value.length();
             }
 
             config->endGroup();
 
             ret = QDir::fromNativeSeparators(ret);
         }
-#endif // QT_NO_SETTINGS
     }
+#endif // QT_NO_SETTINGS
 
-#ifdef QT_BOOTSTRAPPED
-    // The specs need to be returned verbatim.
-    if (loc == TargetSpecPath || loc == HostSpecPath)
+#ifndef QT_BUILD_QMAKE_BOOTSTRAP
+    if (!fromConf) {
+        const char * volatile path = 0;
+        if (loc == PrefixPath) {
+            path =
+# ifdef QT_BUILD_QMAKE
+                (group != DevicePaths) ?
+                    QT_CONFIGURE_EXT_PREFIX_PATH :
+# endif
+                    QT_CONFIGURE_PREFIX_PATH;
+        } else if (unsigned(loc) <= sizeof(qt_configure_str_offsets)/sizeof(qt_configure_str_offsets[0])) {
+            path = qt_configure_strs + qt_configure_str_offsets[loc - 1];
+#ifndef Q_OS_WIN // On Windows we use the registry
+        } else if (loc == SettingsPath) {
+            path = QT_CONFIGURE_SETTINGS_PATH;
+#endif
+# ifdef QT_BUILD_QMAKE
+        } else if (loc == HostPrefixPath) {
+            path = QT_CONFIGURE_HOST_PREFIX_PATH;
+# endif
+        }
+
+        if (path)
+            ret = QString::fromLocal8Bit(path);
+    }
+#endif
+
+#ifdef QT_BUILD_QMAKE
+    // These values aren't actually paths and thus need to be returned verbatim.
+    if (loc == TargetSpecPath || loc == HostSpecPath || loc == SysrootifyPrefixPath)
         return ret;
 #endif
 
     if (!ret.isEmpty() && QDir::isRelativePath(ret)) {
         QString baseDir;
-#ifdef QT_BOOTSTRAPPED
+#ifdef QT_BUILD_QMAKE
         if (loc == HostPrefixPath || loc == PrefixPath || loc == SysrootPath) {
             // We make the prefix/sysroot path absolute to the executable's directory.
             // loc == PrefixPath while a sysroot is set would make no sense here.
             // loc == SysrootPath only makes sense if qmake lives inside the sysroot itself.
-            baseDir = QFileInfo(qt_libraryInfoFile()).absolutePath();
+            baseDir = QFileInfo(qmake_libraryInfoFile()).absolutePath();
         } else if (loc > SysrootPath && loc <= LastHostPath) {
             // We make any other host path absolute to the host prefix directory.
             baseDir = rawLocation(HostPrefixPath, group);
+        } else {
+            // we make any other path absolute to the prefix directory
+            baseDir = rawLocation(PrefixPath, group);
+        }
 #else
         if (loc == PrefixPath) {
             if (QCoreApplication::instance()) {
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
                 CFBundleRef bundleRef = CFBundleGetMainBundle();
                 if (bundleRef) {
                     QCFType<CFURLRef> urlRef = CFBundleCopyBundleURL(bundleRef);
                     if (urlRef) {
                         QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
-#ifdef Q_OS_MACX
+#ifdef Q_OS_OSX
                         QString bundleContentsDir = QString(path) + QLatin1String("/Contents/");
                         if (QDir(bundleContentsDir).exists())
                             return QDir::cleanPath(bundleContentsDir + ret);
 #else
                         return QDir::cleanPath(QString(path) + QLatin1Char('/') + ret); // iOS
-#endif
+#endif // Q_OS_OSX
                     }
                 }
-#endif
+#endif // Q_OS_DARWIN
                 // We make the prefix path absolute to the executable's directory.
                 baseDir = QCoreApplication::applicationDirPath();
             } else {
                 baseDir = QDir::currentPath();
             }
-#endif
         } else {
             // we make any other path absolute to the prefix directory
-            baseDir = rawLocation(PrefixPath, group);
+            baseDir = location(PrefixPath);
         }
+#endif // QT_BUILD_QMAKE
         ret = QDir::cleanPath(baseDir + QLatin1Char('/') + ret);
     }
     return ret;
@@ -551,16 +654,18 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
 
 QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
 {
-#if !defined(QT_BOOTSTRAPPED) && !defined(QT_NO_SETTINGS)
+#if !defined(QT_BUILD_QMAKE) && !defined(QT_NO_SETTINGS)
     QScopedPointer<const QSettings> settings(QLibraryInfoPrivate::findConfiguration());
     if (!settings.isNull()) {
-        QString key = QLatin1String(platformsSection);
-        key += QLatin1Char('/');
-        key += platformName;
-        key += QLatin1String("Arguments");
+        const QString key = QLatin1String(platformsSection)
+                + QLatin1Char('/')
+                + platformName
+                + QLatin1String("Arguments");
         return settings->value(key).toStringList();
     }
-#endif // !QT_BOOTSTRAPPED && !QT_NO_SETTINGS
+#else
+    Q_UNUSED(platformName);
+#endif // !QT_BUILD_QMAKE && !QT_NO_SETTINGS
     return QStringList();
 }
 
@@ -606,17 +711,15 @@ extern "C" void qt_core_boilerplate();
 void qt_core_boilerplate()
 {
     printf("This is the QtCore library version " QT_BUILD_STR "\n"
-           "Copyright (C) 2015 Digia Plc and/or its subsidiary(-ies).\n"
-           "Contact: http://www.qt-project.org/legal\n"
+           "Copyright (C) 2016 The Qt Company Ltd.\n"
+           "Contact: http://www.qt.io/licensing/\n"
            "\n"
-           "Build date:          %s\n"
            "Installation prefix: %s\n"
            "Library path:        %s\n"
            "Include path:        %s\n",
-           qt_configure_installation + 12,
-           qt_configure_prefix_path_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::PrefixPath] + 12,
-           qt_configure_prefix_path_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::LibrariesPath] + 12,
-           qt_configure_prefix_path_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::HeadersPath] + 12);
+           qt_configure_prefix_path_str + 12,
+           qt_configure_strs + qt_configure_str_offsets[QT_PREPEND_NAMESPACE(QLibraryInfo)::LibrariesPath - 1],
+           qt_configure_strs + qt_configure_str_offsets[QT_PREPEND_NAMESPACE(QLibraryInfo)::HeadersPath - 1]);
 
     QT_PREPEND_NAMESPACE(qDumpCPUFeatures)();
 
