@@ -1,47 +1,60 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qgenericunixservices_p.h"
+#include <QtGui/private/qtguiglobal_p.h>
 
-#include <QtCore/QStandardPaths>
-#include <QtCore/QProcess>
-#include <QtCore/QUrl>
 #include <QtCore/QDebug>
+#include <QtCore/QFile>
+#if QT_CONFIG(process)
+# include <QtCore/QProcess>
+#endif
+#include <QtCore/QSettings>
+#include <QtCore/QStandardPaths>
+#include <QtCore/QUrl>
 
 #include <stdlib.h>
 
 QT_BEGIN_NAMESPACE
 
+#if QT_CONFIG(multiprocess)
+
 enum { debug = 0 };
 
 static inline QByteArray detectDesktopEnvironment()
@@ -60,11 +73,29 @@ static inline QByteArray detectDesktopEnvironment()
         return QByteArrayLiteral("MATE");
 
     // Fallback to checking $DESKTOP_SESSION (unreliable)
-    const QByteArray desktopSession = qgetenv("DESKTOP_SESSION");
+    QByteArray desktopSession = qgetenv("DESKTOP_SESSION");
+
+    // This can be a path in /usr/share/xsessions
+    int slash = desktopSession.lastIndexOf('/');
+    if (slash != -1) {
+#ifndef QT_NO_SETTINGS
+        QSettings desktopFile(QFile::decodeName(desktopSession + ".desktop"), QSettings::IniFormat);
+        desktopFile.beginGroup(QStringLiteral("Desktop Entry"));
+        QByteArray desktopName = desktopFile.value(QStringLiteral("DesktopNames")).toByteArray();
+        if (!desktopName.isEmpty())
+            return desktopName;
+#endif
+
+        // try decoding just the basename
+        desktopSession = desktopSession.mid(slash + 1);
+    }
+
     if (desktopSession == "gnome")
         return QByteArrayLiteral("GNOME");
-    if (desktopSession == "xfce")
+    else if (desktopSession == "xfce")
         return QByteArrayLiteral("XFCE");
+    else if (desktopSession == "kde")
+        return QByteArrayLiteral("KDE");
 
     return QByteArrayLiteral("UNKNOWN");
 }
@@ -96,13 +127,14 @@ static inline bool detectWebBrowser(const QByteArray &desktop,
     if (desktop == QByteArray("KDE")) {
         // Konqueror launcher
         if (checkExecutable(QStringLiteral("kfmclient"), browser)) {
-            browser->append(QStringLiteral(" exec"));
+            browser->append(QLatin1String(" exec"));
             return true;
         }
     } else if (desktop == QByteArray("GNOME") ||
                desktop == QByteArray("LXDE") ||
                desktop == QByteArray("LXQT") ||
                desktop == QByteArray("MATE") ||
+               desktop == QByteArray("X-CINNAMON") ||
                desktop == QByteArray("XFCE")) {
         if (checkExecutable(QStringLiteral("gvfs-open"), browser))
             return true;
@@ -119,8 +151,8 @@ static inline bool launch(const QString &launcher, const QUrl &url)
     const QString command = launcher + QLatin1Char(' ') + QLatin1String(url.toEncoded());
     if (debug)
         qDebug("Launching %s", qPrintable(command));
-#if defined(QT_NO_PROCESS)
-    const bool ok = ::system(qPrintable(command + QStringLiteral(" &")));
+#if !QT_CONFIG(process)
+    const bool ok = ::system(qPrintable(command + QLatin1String(" &")));
 #else
     const bool ok = QProcess::startDetached(command);
 #endif
@@ -141,7 +173,7 @@ bool QGenericUnixServices::openUrl(const QUrl &url)
         return openDocument(url);
 
     if (m_webBrowser.isEmpty() && !detectWebBrowser(desktopEnvironment(), true, &m_webBrowser)) {
-        qWarning("%s: Unable to detect a web browser to launch '%s'", Q_FUNC_INFO, qPrintable(url.toString()));
+        qWarning("Unable to detect a web browser to launch '%s'", qPrintable(url.toString()));
         return false;
     }
     return launch(m_webBrowser, url);
@@ -150,10 +182,32 @@ bool QGenericUnixServices::openUrl(const QUrl &url)
 bool QGenericUnixServices::openDocument(const QUrl &url)
 {
     if (m_documentLauncher.isEmpty() && !detectWebBrowser(desktopEnvironment(), false, &m_documentLauncher)) {
-        qWarning("%s: Unable to detect a launcher for '%s'", Q_FUNC_INFO, qPrintable(url.toString()));
+        qWarning("Unable to detect a launcher for '%s'", qPrintable(url.toString()));
         return false;
     }
     return launch(m_documentLauncher, url);
 }
 
+#else
+QByteArray QGenericUnixServices::desktopEnvironment() const
+{
+    return QByteArrayLiteral("UNKNOWN");
+}
+
+bool QGenericUnixServices::openUrl(const QUrl &url)
+{
+    Q_UNUSED(url)
+    qWarning("openUrl() not supported on this platform");
+    return false;
+}
+
+bool QGenericUnixServices::openDocument(const QUrl &url)
+{
+    Q_UNUSED(url)
+    qWarning("openDocument() not supported on this platform");
+    return false;
+}
+
+#endif // QT_NO_MULTIPROCESS
+
 QT_END_NAMESPACE
